#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const parseWebpackStats_1 = require("./parseWebpackStats");
const invokeWebpack_1 = require("./invokeWebpack");
const fs_1 = require("fs");
const ardoq_sdk_js_1 = require("ardoq-sdk-js");
const CACHE_FILE = './stats-cache.json';
const WEBPACK_CONFIG = 'webpack.config.js';
const getOptionsFromArgs = (args) => {
    const options = {
        dryRun: false,
        clearCache: false,
    };
    args.forEach(arg => {
        if (arg === 'dryRun') {
            options.dryRun = true;
        }
        else if (arg === 'clearCache') {
            options.clearCache = true;
        }
    });
    return options;
};
function loadConfig(path) {
    console.log('Reading config', path);
    const config = JSON.parse(fs_1.readFileSync(path, 'utf8'));
    if (process.env.ARDOQ_TOKEN) {
        console.log('Using environment token');
        config.api.authToken = process.env.ARDOQ_TOKEN;
    }
    return config;
}
function readWebpackModules(dir, clearCache) {
    if (!clearCache) {
        try {
            console.log('Reading webpack modules from cache');
            return JSON.parse(fs_1.readFileSync(CACHE_FILE, 'utf8'));
        }
        catch (e) {
            console.log('Failed to read webpack modules from cache', e.message);
        }
    }
    console.log('Extracting webpack modules');
    const modules = invokeWebpack_1.invokeWebpack(dir, WEBPACK_CONFIG);
    console.log('Saving webpack modules to cache');
    fs_1.writeFileSync(CACHE_FILE, JSON.stringify(modules));
    return modules;
}
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('Running Webpack-to-Ardoq version 0.2.0');
        if (process.argv.length < 4) {
            console.error('Illegal usage');
            return;
        }
        const dir = process.argv[2];
        const config = loadConfig(process.argv[3]);
        const options = getOptionsFromArgs(process.argv.slice(4));
        try {
            const modules = readWebpackModules(dir, options.clearCache);
            console.log('Parsing graph');
            const graph = parseWebpackStats_1.extractGraph(config.simplifications.map(ptrn => new RegExp(ptrn)), dir, modules);
            console.log('Syncing to Ardoq');
            yield ardoq_sdk_js_1.sync(config.api, config.workspaces, graph, [
                {
                    name: 'wtaExtension',
                    label: '[WTA] File extension',
                    type: ardoq_sdk_js_1.FieldType.TEXT,
                },
                {
                    name: 'wtaSize',
                    label: '[WTA] File size',
                    type: ardoq_sdk_js_1.FieldType.NUMBER,
                },
                {
                    name: 'wtaDepth',
                    label: '[WTA] File depth',
                    type: ardoq_sdk_js_1.FieldType.NUMBER,
                },
            ]);
            console.log('DONE');
        }
        catch (err) {
            console.log('ERR:', err);
            if (err instanceof ardoq_sdk_js_1.RequestError) {
                console.log(JSON.stringify(err.requestBody, undefined, 2));
            }
            process.exit(1);
        }
    });
}
if (require.main === module) {
    main();
}
