"use strict";
/**
 * Parses a webpack stats json file into a usable format for analyzing files
 * and import dependencies.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// TODO - Use actual streams, instead of iterating over an array 100 times
const path_1 = __importDefault(require("path"));
const names_1 = require("./names");
function flatten(array) {
    return array.reduce((acc, element) => element instanceof Array ? [...acc, ...element] : [...acc, element], []);
}
function normalizePath(simplifications, projectPath, filePath) {
    for (const ptrn of simplifications) {
        const match = ptrn.exec(filePath);
        if (match) {
            return match[1];
        }
    }
    const absolutePath = path_1.default.resolve(projectPath, 'src', 'js', 'ardoq', filePath);
    return path_1.default.relative(projectPath, absolutePath);
}
const extensionModelTypeMap = new Map([
    ['.js', names_1.ComponentType.JAVASCRIPT],
    ['.jsx', names_1.ComponentType.JAVASCRIPT],
    ['.ts', names_1.ComponentType.TYPESCRIPT],
    ['.tsx', names_1.ComponentType.TYPESCRIPT],
    ['.less', names_1.ComponentType.STYLE],
    ['.css', names_1.ComponentType.STYLE],
    ['.hbs', names_1.ComponentType.TEMPLATE],
    ['.png', names_1.ComponentType.GRAPHIC],
    ['.svg', names_1.ComponentType.GRAPHIC],
    ['.eot', names_1.ComponentType.FONT],
    ['.ttf', names_1.ComponentType.FONT],
    ['.woff', names_1.ComponentType.FONT],
    ['.woff2', names_1.ComponentType.FONT],
    ['.0', names_1.ComponentType.FONT],
]);
const referenceTypeMap = new Map([
    ['harmony', names_1.ReferenceType.HARMONY],
    ['cjs', names_1.ReferenceType.COMMON_JS],
]);
function extractModuleType(file, isLib) {
    if (isLib)
        return names_1.ComponentType.LIBRARY;
    return extensionModelTypeMap.get(path_1.default.extname(file)) || names_1.ComponentType.UNKNOWN;
}
function extractReferenceType(wrModule) {
    return (referenceTypeMap.get(wrModule.type.split(' ')[0]) || names_1.ReferenceType.UNKNOWN);
}
function isModuleIdInteresting(moduleId) {
    return typeof moduleId === 'string';
}
function isModuleNameInteresting(name) {
    // Exclude '(webpack) ....'
    return name.indexOf('(') !== 0;
}
function getParent(file) {
    const parent = path_1.default.dirname(file);
    if (parent === '.')
        return undefined;
    return parent;
}
const renderComponent = (simplifications) => (wModule) => {
    const isLib = simplifications.some(ptrn => ptrn.test(wModule.path));
    return {
        customId: wModule.path,
        name: path_1.default.basename(wModule.path),
        type: extractModuleType(wModule.path, isLib),
        workspace: isLib ? names_1.Workspace.LIBRARIES : names_1.Workspace.SOURCES,
        parent: getParent(wModule.path),
        fields: {
            wtaExtension: path_1.default.extname(wModule.path),
            wtaSize: wModule.size,
            wtaDepth: wModule.depth,
        },
    };
};
const extractReferences = (simplifications, projectPath) => (target) => {
    // Extracts references from a module's 'reasons'
    // Depends on the fact that the actual import is the earliest reason for each
    // dependency
    return target.reasons
        .filter(m => isModuleIdInteresting(m.moduleId))
        .filter(m => isModuleNameInteresting(m.moduleName))
        .map(m => {
        const p = normalizePath(simplifications, projectPath, m.moduleName);
        const type = extractReferenceType(m);
        return {
            customId: `${type}:${p}:${target.path}`,
            type,
            source: p,
            target: target.path,
        };
    })
        .filter(ref => !(simplifications.some(ptrn => ptrn.test(ref.source)) &&
        simplifications.some(ptrn => ptrn.test(ref.target))));
};
function extractDirectories(comp) {
    return comp.customId
        .split('/')
        .reduce((acc, name, index, array) => {
        if (index === array.length - 1)
            return acc; // Skip the file name
        const isLib = comp.type === names_1.ComponentType.LIBRARY;
        const p = array.slice(0, index + 1).join('/');
        return [
            ...acc,
            {
                customId: p,
                name: name,
                workspace: isLib ? names_1.Workspace.LIBRARIES : names_1.Workspace.SOURCES,
                parent: getParent(p),
                type: names_1.ComponentType.DIRECTORY,
            },
        ];
    }, []);
}
/**
 * Extracts a graph of modules in a useful format from webpack stats object.
 */
function extractGraph(simplifications, projectPath, rawModules) {
    const modulesArray = rawModules
        .filter(m => isModuleIdInteresting(m.id) && isModuleNameInteresting(m.name))
        .map(m => (Object.assign(Object.assign({}, m), { path: normalizePath(simplifications, projectPath, m.name) })));
    const references = flatten(modulesArray.map(extractReferences(simplifications, projectPath))).filter((ref, _, array) => array.find(b => ref.customId === b.customId) === ref);
    const modules = modulesArray
        .map(renderComponent(simplifications))
        .filter(comp => comp.fields.wtaDepth === 1 ||
        references.some(i => i.target === comp.customId || i.source === comp.customId));
    const directories = flatten(modules.map(extractDirectories)).filter((dir, _, array) => array.find(b => dir.customId === b.customId) === dir);
    const components = [...directories, ...modules].filter((comp, _, array) => array.find(b => comp.customId === b.customId) === comp);
    return {
        components,
        references,
    };
}
exports.extractGraph = extractGraph;
